use std::io;
use std::sync::Arc;
use tokio::io::AsyncWriteExt;
use tokio::net::TcpListener;
use tokio::sync::RwLock;

const FLAG: u8 = 0x7e;

#[tokio::main]
async fn main() {
    apiserver().await.unwrap();
    loop {
        std::thread::sleep(std::time::Duration::from_secs(1));
    }
}

pub async fn apiserver() -> io::Result<()> {
    let addr = "0.0.0.0:7700";
    let listener = match TcpListener::bind(addr).await {
        Ok(o) => o,
        Err(e) => {
            println!("error: {e}");
            std::process::exit(1);
        }
    };

    tokio::spawn(async move {
        loop {
            match listener.accept().await {
                Ok((mut socket, _addr)) => {
                    let mut buf = vec![0; 256];

                    match socket.readable().await {
                        Ok(_) => {
                            match socket.try_read(&mut buf) {
                                Ok(_) => {}
                                Err(e) => {
                                    println!("error: {e}");
                                    continue;
                                }
                            };
                        }
                        Err(e) => {
                            println!("error: {e}");
                            continue;
                        }
                    }

                    let data = buf.to_vec();
                    let msg = parse_msg(data);
                    println!("{:X?}", msg);
                }
                Err(err) => {
                    println!("error: {err}");
                }
            }
        }
    });
    Ok(())
}

fn parse_msg(data: Vec<u8>) -> Message {
    if data[0] == FLAG {
        println!("first flag ok");
    };

    let mut msg: Message = Message::default();

    msg.Header.Id = msg.Header.Id + (data[1] << 1) as u16 + data[2] as u16;
    msg.Header.Properties = msg.Header.Properties + (data[3] << 1) as u16 + data[4] as u16;
    for (i, v) in data[5..11].iter().enumerate() {
        msg.Header.TerminalId[i] = *v;
    }
    msg.Header.SerialNumber = msg.Header.SerialNumber + (data[11] << 1) as u16 + data[12] as u16;

    let endbody = 13 + msg.Header.Properties as usize;
    let range: std::ops::Range<usize> = 13..(endbody);
    let check_idx = 13 + msg.Header.Properties as usize;
    let endflag: u8 = check_idx as u8 + 1;

    for i in data[range].iter() {
        msg.Body.push(*i);
    }

    msg.Check = data[check_idx];

    if data[endflag as usize] == FLAG {
        println!("end flag ok");
    } else {
        println!("error with endflag");
    }
    msg
}

#[derive(Default, Debug)]
pub struct Message {
    pub Header: MessageHeader,
    pub Body: Vec<u8>,
    pub Check: u8,
}

#[derive(Default, Debug)]
pub struct MessageHeader {
    pub Id: u16,
    pub Properties: u16,
    pub TerminalId: [u8; 6],
    pub SerialNumber: u16,
}
